/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// graphe non orienté

// implémentation par listes d'adjacence

#include "prelude.h"
#include "list.h"
#include "graph.h"

struct Graph {
  int nbv;
  list **adj;
};

graph *graph_create(int size) {
  assert(size >= 0);
  graph *g = malloc(sizeof(struct Graph));
  g->nbv = size;
  g->adj = calloc(size, sizeof(list));
  for (int i = 0; i < size; i++) {
    g->adj[i] = NULL;
  }
  return g;
}
int graph_size(graph *g) {
  return g->nbv;
}
bool graph_is_vertex(graph *g, int u) {
  return 0 <= u && u < g->nbv;
}
void graph_add_edge(graph *g, int u, int v) {
  assert(graph_is_vertex(g, u) && graph_is_vertex(g, v));
  g->adj[u] = list_cons(v, g->adj[u]);
  g->adj[v] = list_cons(u, g->adj[v]);
}
bool graph_has_edge(graph *g, int u, int v) {
  assert(graph_is_vertex(g, u) && graph_is_vertex(g, v));
  for (list *l = g->adj[u]; l != NULL; l = l->next) {
    if (l->value == v) {
      return true;
    }
  }
  return false;
}
list *graph_succ(graph *g, int u) {
  assert(graph_is_vertex(g, u));
  return g->adj[u];
}
int graph_nbe(graph *g) {
  int e = 0;
  for (int i = 0; i < g->nbv; i++)
    for (list *l = g->adj[i]; l != NULL; l = l->next)
      if (l->value > i) {
        e++;
      }
  return e;
}
void graph_delete(graph *g) {
  for (int i = 0; i < g->nbv; i++) {
    list_delete(g->adj[i]);
  }
  free(g);
}

graph *graph_load(char *file) {
  FILE *f = fopen(file, "r");
  int n;
  if (fscanf(f, "%d\n", &n) == EOF) { fprintf(stderr, "error\n"); exit(1); }
  graph *g = graph_create(n);
  int e;
  if (fscanf(f, "%d\n", &e) == EOF) { fprintf(stderr, "error\n"); exit(1); }
  while (e-- > 0) {
    int i, j;
    if (fscanf(f, "%d %d\n", &i, &j) == EOF) { fprintf(stderr, "error\n"); exit(1); }
    graph_add_edge(g, i, j);
  }
  fclose(f);
  return g;
}

void graph_print(graph *g) {
  printf("%d\n", g->nbv);
  int e = graph_nbe(g);
  printf("%d\n", e);
  for (int i = 0; i < g->nbv; i++) {
    for (list *l = g->adj[i]; l != NULL; l = l->next) {
      if (i < l->value) {
        printf("%d %d\n", i, l->value);
      }
    }
  }
}
